function [xbest,fbest,convergence] = run_fmincon_on_brushless_DC_wheel_motor_benchmark(runs)
% function to run Matlab fmincon with brushless DC wheel motor benchmark
% by Stephane BRISSET, last revision 21/08/2015
% !!!!!!!  requires brushless_DC_wheel_motor_function & moteur_roue.m files  !!!!!!!

lb = [ 0.9 ; 0.5  ; 0.6 ; 150e-3 ; 2e6 ]; % lower bound for Bd, Be, Bcs, Ds, J
ub = [ 1.8 ; 0.76 ; 1.6 ; 330e-3 ; 5e6 ]; % upper bound for Bd, Be, Bcs, Ds, J

convergence = 0; % convergence rate
fbest = Inf; % no solution found yet
xbest = NaN; % no solution found yet

for i=1:runs % multi-start
    
    x0 = rand(size(lb)).*(ub-lb)+lb; % random starting point
    
    % run fmincon with SQP algorithm
    options = optimoptions('fmincon','Algorithm','sqp','Display','off');
    [xopt,fopt,exitflag]=fmincon(@myfun,x0,[],[],[],[],lb,ub,@mycon,options);
    
    if exitflag > 0 % convergence criteria fulfilled
        if fopt < fbest-2e-5 % new best solution
            xbest = xopt;
            fbest = fopt;
            convergence = 1;
        elseif fopt < fbest+2e-5 % same best solution
            convergence = convergence+1;
        end
    end
    
end

convergence = convergence/i;

% nested functions

    function f = myfun(x)
        f = brushless_DC_wheel_motor_function(x,false);
    end

    function [g,h] = mycon(x)
        [~,g] = brushless_DC_wheel_motor_function(x,false);
        h = []; % no equality constraint
    end

end